function dxdt = ODE_set(t,variables,parameters,constants,input_data)
%% define state variables
G_Gut = variables(1);      %glucose mass in gut (mg)
G_Pl = variables(2);       %plasma glucose concentration (mmol/L)
Del_G_Eff = variables(3);  %delayed glucose effect on insulin secretion (unitless)
I_Pl = variables(4);       %plasma insulin concentration (mU/L)
Del_I = variables(5);      %delayed insulin concentration (mU/L)

%% model constants
c = constants;
c_G = c.f_G; % conversion factor glucose - convert mg to mmol (mmol/mg)
c_I = c.f_I; % conversion factor insulin - convert pmol to mU (mU/pmol)
V_G = c.V_G; % glucose distribution volume (L)
V_I = c.V_I; % insulin distribution volume (L)
r_t = c.r_t; % threshold for renal glucose extraction (mmol/L)
r_f = c.r_f; % glomerular filtration rate (L/min)

%% model input
D_meal = input_data.meal.G; %total amount of carbohydrates ingested (mg)

%% model parameters
g1    = parameters(1); % rate constant for fast glucose appearance in gut [1/min]
g2    = parameters(2); % rate constant for slow glucose appearance in gut [1/min]
g3    = parameters(3); % rate constant for gut emptying [1/min]
g4    = parameters(4); % effect function parameter of insulin on EGP [unitless]
g5    = parameters(5); % effect function parameter of glucose on EGP [unitless]
g6    = parameters(6); % effect function parameter of glucose on glucose uptake [unitless]
g7    = parameters(7); % effect function parameter of insulin on glucose uptake [unitless]
g8    = parameters(8); % effect function parameter of glucose concentration on insulin secretion [unitless]
g9    = parameters(9); % effect function parameter of glucose derivative on insulin secretion [unitless]

sigma = parameters(10); % shape factor for the appearance of glucose intake in gut [unitless]

G_ref  = parameters(11); % reference plasma glucose [mmol/L]
I_ref  = parameters(12); % reference plasma insulin [mU/L]
IS_ref = parameters(13); % reference insulin secretion [mU/min]
U_ii = parameters(14);   % insulin independent glucose uptake [mmol/min]
Uid_ref = parameters(15);% reference insulin dependent glucose uptake [mmol/min]

G_DelT = parameters(16); % delay time of glucose concentration effect on insulin secretion (min)
I_DelT = parameters(17); % delay time of insulin (min)

mid1 = parameters(18);   % effect function parameter of glucose concentration on insulin secretion [unitless]
mid2 = parameters(19);   % effect function parameter of glucose derivative on insulin secretion [unitless]

w_IonEGP = parameters(20);  %weight of insulin's effect on EGP (endogeneous glucose production) [unitless]
w_GonEGP = parameters(21);  %weight of glucose's effect on EGP (endogeneous glucose production) [unitless]
w_GonUid = parameters(22);  %weight of glucose's effect on Uid (insulin dependent glucose uptake to tissues) [unitless]
w_IonUid = parameters(23);  %weight of insulin's effect on Uid (insulin dependent glucose uptake to tissues) [unitless]
w_GonIS = parameters(24);   %weight of glucose's effect on IS  (insulin secretion) [unitless]
w_GDeronIS = parameters(25);%weight of glucose derivative's effect on IS  (insulin secretion) [unitless]

bottom1 = parameters(26); %effect function parameter of glucose concentration on insulin secretion [unitless]
bottom2 = parameters(27); %effect function parameter of glucose derivative on insulin secretion [unitless]

GDer_ref = parameters(28); %reference glucose derivative [mmol/L/min]

top1 = (1+exp((mid1-1)./g8)).*w_GonIS; %effect function parameter of glucose concentration on insulin secretion [unitless]
top2 = (1+exp(mid2./g9)).*w_GDeronIS;  %effect function parameter of glucose derivative on insulin secretion [unitless]

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Model equations
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Conversion of input parameters

Uid_ref = Uid_ref / V_G;  % [mmol/L/min]
U_ii = U_ii / V_G;        % [mmol/L/min]
IS_ref = IS_ref / V_I;    % [mU/L/min]

EGP_ref = U_ii + Uid_ref; % [mmol/L/min]

%% Gut

% glucose appearance in gut from meal
G_meal  = (sigma.*(g1.^sigma).*(t.^(sigma-1)).*exp(-((g1.*t).^sigma)).*exp(-(g2.*t)).*D_meal); 
% gut emptying rate of glucose into plasma
G_empt = g3.*G_Gut;
% glucose mass in gut
D.G_Gut   = G_meal - G_empt;

%% Plasma glucose

% glucose appearance from gut, unit conversion from (mg/min) to (mmol/L/min)
G_fromGut = G_empt.*c_G./V_G;

% endogeneous glucose production (EGP) (mmol/L/min)
InsEffonEGP = g4.*exp(log(w_IonEGP./g4).*(Del_I./I_ref));
GluEffonEGP = g5.*exp(log(w_GonEGP./g5).*(G_Pl./G_ref));
G_EGP = EGP_ref.*(InsEffonEGP+GluEffonEGP);

% glucose uptake by the tissues (mmol/L/min)
GluEffonUid = w_GonUid.*g6.*((G_Pl./G_ref)./(g6-1+(G_Pl./G_ref)));
InsEffonUid = (w_IonUid-g7)+g7.*(Del_I./I_ref);
U_id = Uid_ref.*GluEffonUid.*InsEffonUid; 
G_Upt = U_id + U_ii;

% renal extraction of plasma glucose (mmol/L/min)
G_Ren = ((r_f./V_G).*(G_Pl - r_t)).*(G_Pl > r_t);

% rate of change of plasma glucose (mmol/L/min)
D.G_Pl = G_fromGut + G_EGP - G_Upt - G_Ren;

%% Plasma insulin

% insulin secretion [mU/L/min]
GluEffonIS = bottom1+((top1-bottom1)./(1+exp((mid1-(G_Pl./G_ref))./g8))); 
D.Del_G_Eff = (GluEffonIS-Del_G_Eff)./G_DelT; 
GluDer = G_fromGut + G_EGP - G_Upt - G_Ren; 
GluDerEffonIS = bottom2+((top2-bottom2)./(1+exp((mid2-(GluDer./(GDer_ref+1)))./g9)));
I_pnc = IS_ref.*(GluEffonIS+Del_G_Eff+GluDerEffonIS);

% insulin degradation [mU/L/min]
DegrFr = IS_ref./I_ref; %[1/min]
I_degr = DegrFr.*I_Pl;

% rate of change of plasma insulin [mU/L/min]
D.I_Pl = I_pnc - I_degr;

% delayed plasma insulin [mU/L]
D.Del_I = (I_Pl-Del_I)./I_DelT;

%% Output of differential equations
dxdt=[D.G_Gut; D.G_Pl; D.Del_G_Eff; D.I_Pl; D.Del_I];
